﻿

#include "VsxProtocolDriverDemo.h"

#include <PF.VsxProtocolDriver.WrapperNE.h>

#include <cstdio>
#include <string>



void print_error(VsxStatusCode code)
{
    const char* error_text = nullptr;
    VsxStatusCode ret;
    ret = vsx_GetErrorText(code, &error_text);
    if (ret == VSX_STATUS_SUCCESS)
    {
        printf("Error code %s\n", error_text);
        ret = vsx_ReleaseString(&error_text);
    }
}


int main(int argc, char** argv)        // The instance to the async VsxProtocolDriver
{
    //First discover devices via UDP and use the ip of the first device found

    VsxDeviceList* devList = nullptr;
    VsxStatusCode ret;
    VsxSystemHandle* vsx = nullptr;

    printf("VsxProtocolDriver demo\n\n");

    printf("Probing for sensors via UDP ....\n");

    ret = vsx_GetUdpDeviceList(&devList);

    if (ret != VSX_STATUS_SUCCESS)
    {
        print_error(ret);
        return -1;
    }


    if (devList->length > 0)
    {
        // create a new VsxProtocolDriver instance
        VsxDevice dev = devList->devices[0];
        printf("Device found: %s %s\n", dev.sensorType, dev.ipAddress);
        ret = vsx_InitTcpSensor(&vsx, dev.ipAddress, "");
    }
    else //use fix ip address
    {
        printf("No device found via UDP. Connecting to 192.168.2.4.\n");
        // create a new VsxProtocolDriver instance with fix ip address
        ret = vsx_InitTcpSensor(&vsx, "192.168.2.4", "");
    }

    // free device list
    vsx_ReleaseDeviceList(&devList);

    if (ret != VSX_STATUS_SUCCESS)
    {
        print_error(ret);
        return -2;
    }

    // Connect with device
    ret = vsx_Connect(vsx);
    if (ret != VSX_STATUS_SUCCESS)
    {
        print_error(ret);
        ret = vsx_ReleaseSensor(&vsx);
        return -3;
    }


    // Get the current device information
    VsxDevice* device = nullptr;
    ret = vsx_GetDeviceInformation(vsx, &device);

    if (ret != VSX_STATUS_SUCCESS)
    {
        print_error(ret);
        vsx_ReleaseSensor(&vsx);
        return -4;
    }

    printf("Actual IP from device %s\n", device->ipAddress);

    std::string sensorType = device->sensorType;

    ret = vsx_ReleaseDevice(&device);
    if (ret != VSX_STATUS_SUCCESS)
    {
        print_error(ret);
        vsx_ReleaseSensor(&vsx);
        return -5;
    }

    if (sensorType == "SR3D_TOF")
    {
        RunExample_Sr3DTof(vsx);
    }
    else if (sensorType == "SR3D_STEREO")
    {
        RunExample_Sr3DStereo(vsx);

    }
    else if (sensorType == "SMARTRUNNER")
    {
        RunExample_Sr2D(vsx);
    }
    else
    {
        printf("Unknown sensor type: %s", device->sensorType);
    }

    // Disconnect device
    ret = vsx_Disconnect(vsx);
    if (ret != VSX_STATUS_SUCCESS)
    {
        print_error(ret);
    }

    // Release sensor instance
    ret = vsx_ReleaseSensor(&vsx);
    if (ret != VSX_STATUS_SUCCESS)
    {
        print_error(ret);
        return -6;
    }

    return 0;

}
