﻿using PF.VsxProtocolDriver.Types;
using System.Threading.Channels;

namespace VsxProtocolDriverDemo
{
    // This example programs connects with the device, gets device informations and all parameters and starts grabbing 3D data
    static class ExampleSr3DStereo
    {
        public static async Task RunExample()
        {
            // Get complete parameter list
            var paramRes = await Program._vsxProtocolDriver.GetParameterList();
            if (paramRes.Succ)
            {
                Console.WriteLine($"--- Device contains the following parameters: ---");
                foreach (var p in paramRes.ParameterList)
                {
                    Console.WriteLine($"Id: {p.ParameterId} , Name: {p.Name}, Value: {p.Value}");
                }
            }
            else
            {
                Console.WriteLine($"Unable to receive parameter list: {paramRes.ErrorDesc?.Message}");
                return;
            }

            Console.WriteLine();
            Console.WriteLine("Press any key to start 3D data grabbing.");
            Console.ReadKey();

            // Start autotrigger and enable trigger, see manual for config Id and parameter Id
            var setParamResult = await Program._vsxProtocolDriver.SetSingleParameterValue(Sr3DParameterIdentifierStereo.TriggerSource.SettingsVersion, Sr3DParameterIdentifierStereo.TriggerSource.ConfigVersion, Sr3DParameterIdentifierStereo.TriggerSource.ConfigId, Sr3DParameterIdentifierStereo.TriggerSource.ParamId, Sr3DParameterIdentifierStereo.TriggerSource.Items[0]);
            setParamResult = await Program._vsxProtocolDriver.SetSingleParameterValue(Sr3DParameterIdentifierStereo.TriggerSource.SettingsVersion, Sr3DParameterIdentifierStereo.TriggerSource.ConfigVersion, Sr3DParameterIdentifierStereo.TriggerEnabled.ConfigId, Sr3DParameterIdentifierStereo.TriggerEnabled.ParamId, 1);

            // Alternative:
            //param = paramRes.ParameterList.Find(x => x.ConfigId == Sr3DParameterIdentifierStereo.TriggerEnabled.ConfigId && x.ParameterId == Sr3DParameterIdentifierStereo.TriggerEnabled.ParamId); // start trigger
            //setParamResult = await Program._vsxProtocolDriver.SetSingleParameterValue(param, "1");

            // Set output format to disparity, then you get the data needed from device
            setParamResult = await Program._vsxProtocolDriver.SetSingleParameterValue(Sr3DParameterIdentifierStereo.TriggerSource.SettingsVersion, Sr3DParameterIdentifierStereo.TriggerSource.ConfigVersion, Sr3DParameterIdentifierStereo.OutputMode.ConfigId, Sr3DParameterIdentifierStereo.OutputMode.ParamId, Sr3DParameterIdentifierStereo.OutputMode.Items[1]);

            // Start listening for 3D data. 3 containers will be buffered internaly
            Program._vsxProtocolDriver.ResetDynamicContainerGrabber(3);
            do
            {
                while (!Console.KeyAvailable)
                {
                    await Consume();
                }
            } while (Console.ReadKey(true).Key != ConsoleKey.Escape);

            // Disconnect device
            var _ = Program._vsxProtocolDriver.Disconnect();

            Console.WriteLine();
            Console.WriteLine("Grabbing finished, device disconnected, press any key to exit.");
            while (!Console.KeyAvailable) ;
        }

        private static async Task<bool> Consume()
        {
            try
            {
                var dataItem = await Program._vsxProtocolDriver.GetDynamicContainer(100);
                if (dataItem.Succ && (dataItem.Container != null))
                {
                    // now point cloud data is inside of the calibratedA, calibratedB and calibratedC messages
                    IVsxData xLayerImage = dataItem.Container.GetMessage("CalibratedA");
                    IVsxData yLayerImage = dataItem.Container.GetMessage("CalibratedB");
                    IVsxData zLayerImage = dataItem.Container.GetMessage("CalibratedC");

                    if (xLayerImage is IVsxImage xImg)
                    {
                        // process point cloud data, p.e., or save it with Save3DPointCloudData
                        int xWidth = xImg.Width;
                        int xHeight = xImg.Height;
                        int xLinePitch = xImg.LinePitch;
                        if (xImg.Format == ImageData2Format.Coord3D_A32f) // The stereo x image format, y is B32f, ...
                        {
                            float[] pixelValuesX = xImg.ImageDataFloats;
                        }

                        Console.WriteLine("3D data grabbed, press escape to stop grabbing");
                    }
                    dataItem.Container.Dispose();
                }
            }
            catch (ChannelClosedException) { }
            return true;
        }
    }
}
