﻿using PF.VsxProtocolDriver;

namespace VsxProtocolDriverDemo
{
    class Program
    {
        // The instance to the async VsxProtocolDriver
        public static IVsxProtocolDriver _vsxProtocolDriver;

        static async Task Main()
        {
            //First discover devices via UDP and use the ip of the first device found
            var sensors = await VsxProtocolDriver.UdpDeviceList();
            if (sensors.Succ && sensors.DeviceList.Count > 0)
            {
                // create a new VsxProtocolDriver instance
                var dev = sensors.DeviceList[0];
                Console.WriteLine($"Device found: {dev.SensorType}({dev.IpAddress})");
                _vsxProtocolDriver = VsxProtocolDriver.InitTcpDevice(sensors.DeviceList[0].IpAddress);
            }
            else //use fix ip address
            {
                // create a new VsxProtocolDriver instance with fix ip address
                _vsxProtocolDriver = VsxProtocolDriver.InitTcpDevice("192.168.2.4");
            }

            // Connect with device
            var connRes = await _vsxProtocolDriver.Connect();
            if (!connRes.Succ)
            {
                Console.WriteLine($"Unable to connect with device: {connRes.ErrorDesc.Message}");
                return;
            }

            // Get the current device information
            var deviceInformationRes = await _vsxProtocolDriver.GetDeviceInformation();
            if (deviceInformationRes.Succ)
            {
                Console.WriteLine($"Connected with device {deviceInformationRes.CurrentDevice.SensorType}, IP: {deviceInformationRes.CurrentDevice.IpAddress}");
                Console.WriteLine();
            }
            else
            {
                Console.WriteLine($"Unable to receive device information: {connRes.ErrorDesc.Message}");
                return;
            }

            switch (deviceInformationRes.CurrentDevice.SensorType)
            {
                case "SR3D_TOF":
                    await ExampleSr3DTof.RunExample();
                    break;
                case "SR3D_STEREO":
                    await ExampleSr3DStereo.RunExample();
                    break;
                case "SMARTRUNNER":
                    await ExampleSr2D.RunExample();
                    break;
            }
        }
    }

    public class ParameterElement
    {
        public ushort SettingsVersion { get; init; } = 1;
        public ushort ConfigVersion { get; init; } = 1;
        public string ConfigId { get; init; } = string.Empty;
        public string ParamId { get; init; } = string.Empty;
        public string[] Items { get; init; } = Array.Empty<string>();
        public string Name { get; init; } = string.Empty;
    }
}